

#include "ansidecl.h"
#include "dis-asm.h"
#include <string.h>


#define FALSE		0
#define TRUE		1

#ifdef T800
#define OPR		0xf
#define PFIX		0x2
#define NFIX		0x6
#define J		0x0
#define CJ		0xA
#define CALL		0x9


#define	FUNCTION_CODE_MASK		0xf0
#define DATA_VALUE_MASK			0x0f
#define MAX_INSTRUCTION_SIZE		10

#define BRANCH_INSTRUCTION(opcode) ((opcode == CJ)||(opcode == J)||(opcode == CALL))

#define READ_MEM(memaddr, insn, size_insn, info )  {\
    int status =\
      (*info->read_memory_func) (memaddr, (bfd_byte *) &insn, size_insn, info);\
    if (status != 0)\
      {\
	(*info->memory_error_func) (status, memaddr, info);\
	return -1;\
      }\
  }\

char *primary_instruction[] = 	{
				"j ",
				"ldlp",
				"pfix",
				"ldnl",
				"ldc",
				"ldnlp",
				"nfix",
				"ldl",
				"adc",
				"call ",
				"cj ",
				"ajw",
				"eqc",
				"stl",
				"stnl",
				"opr"
				};

char *secondary_with_prefix[] = {
				"rev",
				"lb",
				"bsub",
				"endp",
				"diff",
				"add",
				"gcall",
				"in",
				"prod",
				"gt",
				"wsub",
				"out",
				"sub",
				"startp",
				"outbyte",
				"outword",

				"seterr",
				"unknown",
				"resetch",
				"csub0",
				"unknown",
				"stopp",
				"ladd",
				"stlb",
				"sthf",
				"norm",
				"ldiv",
				"ldpi",
				"stlf",
				"xdble",
				"ldpri",
				"rem",

				"ret",
				"lend",
				"ldtimer",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"testerr",
				"testpranal",
				"tin",
				"div",
				"unknown",
				"dist",
				"disc",

				"diss",
				"lmul",
				"not",
				"xor",
				"bcnt",
				"lshr",
				"lshl",
				"lsum",
				"lsub",
				"runp",
				"xword",
				"sb",
				"gajw",
				"savel",
				"saveh",
				"wcnt",

				"shr",
				"shl",
				"mint",
				"alt",
				"altwt",
				"altend",
				"and",
				"enbt",
				"enbc",
				"enbs",
				"move",
				"or",
				"csngl",
				"ccnt1",
				"talt",
				"ldiff",

				"unknown",
				"taltwt",
				"sum",
				"mul",
				"sttimer",
				"stoperr",
				"cword",
				"clrhalterr",
				"sethalterr",
				"testhalterr",
				"dup",
				"move2dinit",
				"move2dall",
				"move2dnonzero",
				"move2dzero",
				"unknown",

				"unknown",
				"unknown",
				"unknown",
				"unpacksn",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"postnormsn",
				"roundsn",
				"unknown",
				"unknown",

				"unknown",
				"ldinf",
				"fmul",
				"cflerr",
				"crcword",
				"crcbyte",
				"bitcnt",
				"bitrevword",
				"bitrevnbits",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",

				"unknown",
				"wsubdb",
				"fpldnldbi",
				"fpchkerr",
				"fpstnldb",
				"unknown",
				"fpldnlsni",
				"fpadd",
				"fpstnlsn",
				"fpsub",
				"fpldnldb",
				"fpmul",
				"fpdiv",
				"unknown",
				"fpldnlsn",
				"fpremfirst",

				"fpremstep",
				"fpnan",
				"fpordered",
				"fpnotfinite",
				"fpgt",
				"fpeq",
				"fpi32tor32",
				"unknown",
				"fpi32tor64",
				"unknown",
				"fpb32tor64",
				"unknown",
				"fptesterr",
				"fprtoi32",
				"fpstnli32",
				"fpldzerosn",

				"fpldzerodb",
				"fpint",
				"unknown",
				"fpdup",
				"fprev",
				"unknown",
				"fpldnladddb",
				"unknown",
				"fpldnlmuldb",
				"unknown",
				"fpldnladdsn",
				"fpentry",
				"fpldnlmulsn",
				"unknown",
				"unknown",
				"unknown",

				};

/* Print one instruction from MEMADDR on INFO->STREAM.
*/

int
print_insn_t800 (memaddr, info)
     bfd_vma memaddr;
     disassemble_info *info;
{
	FILE *stream = info->stream;
	unsigned char instn;
	unsigned char opcode;
	long instruction_code;
	char complete;
	long data_val;
	int instruction_size;
	bfd_vma cur_addr;

	cur_addr = memaddr;
	instruction_size = 0;
	data_val = 0;
	do {
		READ_MEM(cur_addr, instn, 1, info ) ; instruction_size++; cur_addr++;
		opcode = ( instn & FUNCTION_CODE_MASK ) >> 4;
		if ( opcode == PFIX )
			data_val = ( data_val + ( instn & DATA_VALUE_MASK )) << 4;
		else if ( opcode == NFIX )
			data_val = (~( data_val + ( instn & DATA_VALUE_MASK ))) << 4;
		else  {
			data_val = ( data_val + ( instn & DATA_VALUE_MASK ));
			break;
		}
	} while (( opcode == PFIX)||( opcode == NFIX )) ;

	if ( opcode != OPR ){
		(*info->fprintf_func) ( stream, primary_instruction[ opcode ]);
		if ( BRANCH_INSTRUCTION( opcode )) {
			info->target = cur_addr+data_val;
			(*info->print_address_func) (info->target, info);
		}
		else
			(*info->fprintf_func) ( stream, " %d", data_val );
	}
	else {
		instruction_code = (long)data_val;
		(*info->fprintf_func) ( stream, secondary_with_prefix[ instruction_code ]);
	}
	return( instruction_size );
}

#endif
#ifdef T9000

#define OPR		0xf
#define PFIX		0x2
#define NFIX		0x6
#define J		0x0
#define CJ		0xA
#define CALL		0x9


#define	FUNCTION_CODE_MASK		0xf0
#define DATA_VALUE_MASK			0x0f
#define MAX_INSTRUCTION_SIZE		10

#define BRANCH_INSTRUCTION(opcode) ((opcode == CJ)||(opcode == J)||(opcode == CALL))

#define READ_MEM(memaddr, insn, size_insn, info )  {\
    int status =\
      (*info->read_memory_func) (memaddr, (bfd_byte *) &insn, size_insn, info);\
    if (status != 0)\
      {\
	(*info->memory_error_func) (status, memaddr, info);\
	return -1;\
      }\
  }\

char *primary_instruction[] = 	{
				"j ",
				"ldlp",
				"pfix",
				"ldnl",
				"ldc",
				"ldnlp",
				"nfix",
				"ldl",
				"adc",
				"call ",
				"cj ",
				"ajw",
				"eqc",
				"stl",
				"stnl",
				"opr"
				};

char *secondary_with_prefix[] = {
				"rev",
				"lb",
				"bsub",
				"endp",
				"diff",
				"add",
				"gcall",
				"in",
				"prod",
				"gt",
				"wsub",
				"out",
				"sub",
				"startp",
				"outbyte",
				"outword",

				"unknown",
				"unknown",
				"resetch",
				"csub0",
				"unknown",
				"stopp",
				"ladd",
				"unknown",
				"unknown",
				"norm",
				"ldiv",
				"ldpi",
				"unknown",
				"xdble",
				"ldpri",
				"rem",

				"ret",
				"lend",
				"ldtimer",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"testpranal",
				"tin",
				"div",
				"unknown",
				"dist",
				"disc",

				"diss",
				"lmul",
				"not",
				"xor",
				"bcnt",
				"lshr",
				"lshl",
				"lsum",
				"lsub",
				"runp",
				"xword",
				"sb",
				"gajw",
				"unknown",
				"unknown",
				"wcnt",

				"shr",
				"shl",
				"mint",
				"alt",
				"altwt",
				"altend",
				"and",
				"enbt",
				"enbc",
				"enbs",
				"move",
				"or",
				"csngl",
				"ccnt1",
				"talt",
				"ldiff",

				"unknown",
				"taltwt",
				"sum",
				"mul",
				"sttimer",
				"unknown",
				"cword",
				"unknown",
				"unknown",
				"unknown",
				"dup",
				"move2dinit",
				"move2dall",
				"move2dnonzero",
				"move2dzero",
				"gtu",

				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",

				"unknown",
				"unknown",
				"fmul",
				"unknown",
				"crcword",
				"crcbyte",
				"bitcnt",
				"bitrevword",
				"bitrevnbits",
				"pop",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"ldmemstartval",
				"unknown",

				"unknown",
				"wsubdb",
				"fpldnldbi",
				"unknown",
				"fpstnldb",
				"unknown",
				"fpldnlsni",
				"fpadd",
				"fpstnlsn",
				"fpsub",
				"fpldnldb",
				"fpmul",
				"fpdiv",
				"fprange",
				"fpldnlsn",
				"unknown",

				"unknown",
				"fpnan",
				"fpordered",
				"fpnotfinite",
				"fpgt",
				"fpeq",
				"fpi32tor32",
				"fpge",
				"fpi32tor64",
				"unknown",
				"fpb32tor64",
				"fplg",
				"unknown",
				"fprtoi32",
				"fpstnli32",
				"fpldzerosn",

				"fpldzerodb",
				"fpint",
				"unknown",
				"fpdup",
				"fprev",
				"unknown",
				"fpldnladddb",
				"unknown",
				"fpldnlmuldb",
				"unknown",
				"fpldnladdsn",
				"unknown",
				"fpldnlmulsn",
				"unknown",
				"unknown",
				"unknown",

				"settimeslice",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"ldflags",
				"stflags",
				"xbword",
				"lbx",
				"cb",
				"cbu",
				"insphdr",
				"readbfr",
				"ldconf",
				"stconf",

				"ldcnt",
				"ssub",
				"ldth",
				"ldchstatus",
				"intdis",
				"intenb",
				"unknown",
				"cir",
				"ss",
				"chantype",
				"ls",
				"unknown",
				"ciru",
				"unknown",
				"unknown",
				"fprem",

				"fprn",
				"fpdivby2",
				"fpmulby2",
				"fpsqrt",
				"fprp",
				"fprm",
				"fprz",
				"fpr32tor64",
				"fpr64tor32",
				"fpexpdec32",
				"fpexpinc32",
				"fpabs",
				"unknown",
				"unknown",
				"fpchki32",
				"fpchki64",

				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",
				"unknown",

				"devlb",
				"devsb",
				"devls",
				"devss",
				"devlw",
				"devsw",
				"unknown",
				"unknown",
				"xsword",
				"lsx",
				"cs",
				"csu",
				"unknown",
				"unknown",
				"unknown",
				"unknown"
				};

char *secondary_with_nfix[] = 	{
				"unknown",
				"fpstall",
				"fpldall",
				"stshadow",
				"ldshadow",
				"tret",
				"goprot",
				"selth",
				"syscall",
				"unknown",
				"unknown",
				"wait",
				"signal",
				"timeslice",
				"insertqueue",
				"swaptimer",

				"swapqueue",
				"unknown",
				"stopch",
				"vout",
				"vin",
				"unknown",
				"unknown",
				"swapbfr",
				"sethdr",
				"setchmode",
				"initvclb",
				"writehdr",
				"readhdr",
				"disg",
				"enbg",
				"grant",

				"stmove2dinit",
				"causeerror",
				"unknown",
				"unmkrc",
				"mkrc",
				"irdsq",
				"erdsq",
				"stresptr",
				"ldresptr",
				"unknown",
				"unknown",
				"unknown",
				"devmove",
				"icl",
				"fdcl",
				"ica"

				};

/* Print one instruction from MEMADDR on INFO->STREAM.
*/

int
print_insn_t9000 (memaddr, info)
     bfd_vma memaddr;
     disassemble_info *info;
{
	FILE *stream = info->stream;
	unsigned char instn;
	unsigned char opcode;
	long instruction_code;
	char complete;
	long data_val;
	int instruction_size;
	bfd_vma cur_addr;

	cur_addr = memaddr;
	instruction_size = 0;
	data_val = 0;
	do {
		READ_MEM(cur_addr, instn, 1, info ) ; instruction_size++; cur_addr++;
		opcode = ( instn & FUNCTION_CODE_MASK ) >> 4;
		if ( opcode == PFIX )
			data_val = ( data_val + ( instn & DATA_VALUE_MASK )) << 4;
		else if ( opcode == NFIX )
			data_val = (~( data_val + ( instn & DATA_VALUE_MASK ))) << 4;
		else  {
			data_val = ( data_val + ( instn & DATA_VALUE_MASK ));
			break;
		}
	} while (( opcode == PFIX)||( opcode == NFIX )) ;

	if ( opcode != OPR ){
		(*info->fprintf_func) ( stream, primary_instruction[ opcode ]);
		if ( BRANCH_INSTRUCTION( opcode )) {
			info->target = cur_addr+data_val;
			(*info->print_address_func) (info->target, info);
		}
		else
			(*info->fprintf_func) ( stream, " %d", data_val );
	}
	else {
		instruction_code = (long)data_val;
		if (( instruction_code >= 0 )&&( instruction_code <= 0xFF ))
			(*info->fprintf_func) ( stream, secondary_with_prefix[ instruction_code ]);
		else if (( instruction_code < 0 )&&( instruction_code > -(0x2f)))
			(*info->fprintf_func) ( stream, secondary_with_nfix[ -(instruction_code) ]);
		else  {
			if ( instruction_code == 0x17c )
				(*info->fprintf_func) ( stream, "lddevid" );
			else if ( instruction_code == -( 0x30 ))
				(*info->fprintf_func) ( stream, "fdca" );
			else if ( instruction_code == -( 0x40 ))
				(*info->fprintf_func) ( stream, "nop" );
			else if ( instruction_code == -( 0x84 ))
				(*info->fprintf_func) ( stream, "ldprodid" );
			else 
				(*info->fprintf_func) ( stream, "unknown" );
		}
	}
	return( instruction_size );
}

#endif
